#include <glib.h>

#include <hildon/hildon.h>
#include <hildon/hildon-file-chooser-dialog.h>
#include <location/location-gpsd-control.h>
#include <location/location-gps-device.h>
#include <location/location-misc.h>
#include <location/location-distance-utils.h>

typedef struct {
    HildonProgram *program;
    HildonWindow *window;
    
    GtkWidget * main_vbox;
    GtkWidget *status_label;
    GtkButton *start_button;
} AppData;

static GtkWidget *window = NULL;

static gchar * interface_file_chooser (AppData * appdata, GtkFileChooserAction action)
{
    GtkWidget *dialog;
    gchar *filename = NULL;
    
    dialog = hildon_file_chooser_dialog_new (GTK_WINDOW (appdata->window), action);
    gtk_widget_show_all (GTK_WIDGET (dialog));

    if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK) {
        filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
    }

    gtk_widget_destroy(dialog);
    return filename;
}

static void cb_example_file_save (GtkWidget * w, AppData * data)
{
    gchar *filename = NULL;
    filename = interface_file_chooser (data, GTK_FILE_CHOOSER_ACTION_SAVE);

    if (filename == NULL) {
        filename = "NULL";
    }
    else {
        FILE * f = fopen (filename, "w");
        fprintf (f, "This file was generated by Hildon File Chooser example.");
        fclose (f);
    }

    g_print ("File saved as %s\n", filename);
}

static void
on_gps_device_changed (LocationGPSDevice *device, gpointer data)
{
  GtkLabel *info = (GtkLabel*)data;
  GString *msg;
	if (!device)
		return;

  msg = g_string_sized_new (512);
	if (device->fix) {
		if (device->fix->fields & LOCATION_GPS_DEVICE_TIME_SET) {
			g_print ("time = %f\n", device->fix->time);
      g_string_append_printf(msg, "time = %f\n", device->fix->time);
    }

		if (device->fix->fields & LOCATION_GPS_DEVICE_LATLONG_SET) {
			g_print ("lat = %f, long = %f\n",
					device->fix->latitude,
					device->fix->longitude);
			g_string_append_printf (msg, "lat = %f, long = %f\n",
					device->fix->latitude,
					device->fix->longitude);
    }

		if (device->fix->fields & LOCATION_GPS_DEVICE_ALTITUDE_SET) {
			g_print ("alt = %f\n", device->fix->altitude);
			g_string_append_printf (msg, "alt = %f\n", device->fix->altitude);
    }

		if (device->fix->fields & LOCATION_GPS_DEVICE_SPEED_SET) {
			g_print ("speed = %f\n", device->fix->speed);
			g_string_append_printf (msg, "speed = %f, ", device->fix->speed);
    }

		if (device->fix->fields & LOCATION_GPS_DEVICE_TRACK_SET) {
			g_print ("track = %f\n", device->fix->track);
			g_string_append_printf (msg, "track = %f, ", device->fix->track);
    }

		if (device->fix->fields & LOCATION_GPS_DEVICE_CLIMB_SET) {
			g_print ("climb = %f\n", device->fix->climb);
			g_string_append_printf (msg, "climb = %f\n", device->fix->climb);
    }

		g_print ("Accuracy values:\n");
		g_print ("\tept = %e, eph = %e, epv = %e, epd = %e, "
				"eps = %e, epc = %e\n",
				device->fix->ept,
				device->fix->eph,
				device->fix->epv,
				device->fix->epd,
			       	device->fix->eps,
				device->fix->epc);
	}
	
	g_print ("Satellites in view: %d\n", device->satellites_in_view);
	g_print ("Satellites in use: %d\n", device->satellites_in_use);
  g_string_append_printf (msg, "Satellites = % 2d/% 2d\n", device->satellites_in_use, device->satellites_in_view);
	g_print ("GPS status: %d\n", device->status);
	g_string_append_printf (msg, "GPS status: %d\n", device->status);

  hildon_gtk_window_set_progress_indicator(GTK_WINDOW(window), device->status == LOCATION_GPS_DEVICE_STATUS_NO_FIX);


	if (device->cell_info) {
		if (device->cell_info->flags & LOCATION_CELL_INFO_GSM_CELL_INFO_SET)
			g_print ("Mobile Coutry Code GSM: %d\n", device->cell_info->gsm_cell_info.mcc);

		if (device->cell_info->flags & LOCATION_CELL_INFO_WCDMA_CELL_INFO_SET)
			g_print ("Mobile Coutry Code WCDMA: %d\n", device->cell_info->wcdma_cell_info.mcc);
	}

  gtk_label_set_text(info, msg->str);
  //gtk_widget_show(GTK_WIDGET(info));
  g_string_free(msg, TRUE);
}

static void
on_gps_error (LocationGPSDevice *device, gpointer data)
{
	g_error ("GPS error");
}

static void
on_gps_stop (LocationGPSDevice *device, gpointer data)
{
	g_warning ("GPS stopped");
}

static void
on_gps_start (LocationGPSDevice *device, gpointer data)
{
	g_warning ("GPS started");
}

int main (int argc, char **argv)
{
	GtkWidget *picker_button = NULL;
  AppData * data = g_new0 (AppData, 1);

	hildon_gtk_init (&argc, &argv);
	LocationGPSDControl *control;
	LocationGPSDevice *device;

	data->program = hildon_program_get_instance ();
	g_set_application_name("GPS tracker");

	window = hildon_stackable_window_new ();
	hildon_program_add_window (data->program, HILDON_WINDOW (window));

  data->main_vbox = (void*)gtk_vbox_new(FALSE, 0);

	/* Create a picker button */
	picker_button = hildon_date_button_new (HILDON_SIZE_AUTO,
			HILDON_BUTTON_ARRANGEMENT_VERTICAL);

	/* Set a title to the button*/
	hildon_button_set_title (HILDON_BUTTON (picker_button), "Pick a date");

  data->start_button = GTK_BUTTON (hildon_gtk_button_new (HILDON_SIZE_THUMB_HEIGHT));
  gtk_button_set_label (data->start_button, "Start");
  gtk_box_pack_start(GTK_BOX(data->main_vbox), GTK_WIDGET(data->start_button), FALSE, FALSE, 8);
  gtk_box_pack_start(GTK_BOX(data->main_vbox), GTK_WIDGET(picker_button), FALSE, FALSE, 0);
  data->status_label = gtk_label_new("Hier kommt der Status hin\nUnd hier ist die 2. Zeile");
  gtk_box_pack_start_defaults(GTK_BOX(data->main_vbox), data->status_label);
	/* Add vbox to main window */
	gtk_container_add (GTK_CONTAINER (window), GTK_WIDGET(data->main_vbox));

	g_signal_connect (G_OBJECT (window), "destroy",
			G_CALLBACK (gtk_main_quit), NULL);

	control = location_gpsd_control_get_default ();
  hildon_banner_show_information(GTK_WIDGET(window), NULL, "Hi there!");
	location_gpsd_control_start (control);

	/*
	 * Note that in real life one may want to use some other method and interval
	 * than LOCATION_METHOD_USER_SELECTED and LOCATION_INTERVAL_DEFAULT,
	 * respectively. For more information on possible values for these parameters
	 * please see liblocation online documentation.
	 */
	g_object_set (G_OBJECT (control), 
			"preferred-method", LOCATION_METHOD_USER_SELECTED,
			"preferred-interval", LOCATION_INTERVAL_DEFAULT,
			NULL);

	device  = g_object_new (LOCATION_TYPE_GPS_DEVICE, NULL);

	g_signal_connect (control, "error",		G_CALLBACK (on_gps_error),		NULL);
	g_signal_connect (control, "gpsd-running",	G_CALLBACK (on_gps_start),		NULL);
	g_signal_connect (control, "gpsd-stopped",	G_CALLBACK (on_gps_stop),		NULL);
	g_signal_connect (device,  "changed",		G_CALLBACK (on_gps_device_changed),	data->status_label);

	gtk_widget_show_all (GTK_WIDGET (window));

  hildon_gtk_window_set_progress_indicator(GTK_WINDOW(window), 1);
	gtk_main ();

	location_gpsd_control_stop (control);

	g_object_unref (device);
	g_object_unref (control);
  g_free(data);

	return 0;
}
